#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.ml - 2016 - Compétition ENSAE - Premiers modèles
# 
# Une compétition était proposée dans le cadre du cours *Python pour un Data Scientist* à l'[ENSAE](http://www.ensae.fr/ensae/fr/). Ce notebook facilite la prise en main des données et propose de mettre en oeuvre un modèle logit.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# La première étape constiste à mettre en forme les données pour que les fonctions des modules statsmodels et skitlearn fonctionnent comme on le souhaite

# In[2]:


from pyensae.datasource import download_data
download_data("ensae_competition_2016.zip",
              url="https://github.com/sdpython/ensae_teaching_cs/raw/master/_doc/competitions/2016_ENSAE_2A/")


# In[3]:


import pandas as pd
import statsmodels.api as sm
import pylab as pl
import numpy as np

fichier_train = "./ensae_competition_train.txt"
fichier_test = "./ensae_competition_test_X.txt"

df = pd.read_csv(fichier_train, header=[0,1], sep="\t", index_col=0)


# ## Mise en forme des données pour la régression 
# 
# Pour mieux analyser les données présentes dans la base de données, il faut passer par quelques manipulations.
# 
# Par exemple, il faut transformer les variables SEX, EDUCATION, MARRIAGE en indictatrices afin qu'elles ne soient pas prise en compte comme des variables continues dans le modèle.

# In[4]:


#### Gender dummies
df['X2'] = df['X2'].applymap(str)
gender_dummies = pd.get_dummies(df['X2'] )


# In[5]:


### education dummies
df['X3'] = df['X3'].applymap(str)
educ_dummies = pd.get_dummies(df['X3'] )


# In[6]:


#### marriage dummies
df['X4'] = df['X4'].applymap(str)
mariage_dummies = pd.get_dummies(df['X4'] )


# In[7]:


### On va aussi supprimer les multi index de la table
df.columns = df.columns.droplevel(0)


# In[8]:


#### on aggrège ensuite les 3 tables ensemble
data = df.join(gender_dummies).join(educ_dummies).join(mariage_dummies)
data.head(n=2)


# ## Régression logistique en utilisant statsmodels 
# 
# Pour notre premier modèle, nous allons voir comment fonctionne le module statsmodels
# 
# Par défaut, la regression logit n'a pas de beta zéro pour le module statsmodels : il faut donc l'ajouter

# In[9]:


# première étape pour ce module, il faut ajouter à la main le beta zéro - l'intercept
data['intercept'] = 1.0
data.rename(columns = {'default payment next month' : "Y"}, inplace = True)
data.columns


# In[10]:


# variable = ['AGE', 'BILL_AMT1', 'BILL_AMT2', 'BILL_AMT3', 'BILL_AMT4',
#       'BILL_AMT5', 'BILL_AMT6', 'LIMIT_BAL', 'PAY_0',
#       'PAY_2', 'PAY_3', 'PAY_4', 'PAY_5', 'PAY_6', 'PAY_AMT1', 'PAY_AMT2',
#       'PAY_AMT3', 'PAY_AMT4', 'PAY_AMT5', 'PAY_AMT6', 'SEX_1',
#       'EDUCATION_0', 'EDUCATION_1', 'EDUCATION_2', 'EDUCATION_3',
#       'EDUCATION_4', 'EDUCATION_5', 'MARRIAGE_0', 'MARRIAGE_1',
#       'MARRIAGE_2', 'intercept']

train_cols = ["SEX_1", "AGE", "MARRIAGE_0", 'PAY_0','intercept']


# In[11]:


# Cette cellule n'est nécessaire que si vous utilisez scipy 1.0 avec statsmodels 0.8.
from pymyinstall.fix import fix_scipy10_for_statsmodels08
fix_scipy10_for_statsmodels08()


# In[12]:


logit = sm.Logit(data['Y'], data[train_cols].astype(float))

# fit the model
result = logit.fit()
print(result.summary())


# ## Prédiction sur la base de test
# 
# On va prédire les probabilités de défaut de paiement sur notre base de test.
# 
# Il faut commencer par transformer la base de test comme on l'a fait pour la base de train.

# In[13]:


data_test = pd.read_csv(fichier_test, header=[0,1], sep="\t", index_col=0)


# In[14]:


#### Gender dummies
data_test['X2'] = data_test['X2'].applymap(str)
gender_dummies_test = pd.get_dummies(data_test['X2'] )


# In[15]:


### education dummies
data_test['X3'] = data_test['X3'].applymap(str)
educ_dummies_test = pd.get_dummies(data_test['X3'] )


# In[16]:


#### marriage dummies
data_test['X4'] = data_test['X4'].applymap(str)
mariage_dummies_test = pd.get_dummies(data_test['X4'] )


# In[17]:


### On va aussi supprimer les multi index de la table
data_test.columns = data_test.columns.droplevel(0)


# In[18]:


#### on aggrège ensuite les 3 tables ensemble
data_test = data_test.join(gender_dummies_test).join(educ_dummies_test).join(mariage_dummies_test)


# In[19]:


data_test['intercept'] = 1.0


# In[20]:


data_test[train_cols].head()


# Maintenant que la base de test est également transformée, nous allons appliqué les résultats de notre modèle à cette table en utilisant la fonction predict

# In[21]:


data_test['prediction_statsmodel'] = result.predict(data_test[train_cols])
data_test['prediction_statsmodel'].describe()


# On trouve un taux moyen de défaut de 22%, ce qui est très proche du taux observé dans la base de train

# In[22]:


# puis on l'exporte
data_test['prediction_statsmodel'].to_csv("./answer.csv", index=False)


# ## Regression logistique en utilisant Scikit-learn
# 
# A présent, nous allons utiliser le module scikit learn pour estimer le même modèle que précédemment et comparer les résultats.
# 
# Ici pas besoin d'ajouter une variable égale à 1 (l'intercept) car Scikit learn considère qu'il y a un intercept par défaut.

# In[23]:


from sklearn import linear_model
logistic = linear_model.LogisticRegression()


# In[24]:


print("l'estimation des coefficients", logistic.fit(data[train_cols], data['Y']).coef_, "\n")

print("l'intercept : ", logistic.fit(data[train_cols], data['Y']).intercept_)


# A la différence de statsmodels, Scikit-learn ne propose pas une belle table de résultat : seulement les arrays qui contiennent les coefficients
# 
# Pour le détail des p-value et intervalles de confiance, il faudra les recalculer à la main. 
# 
# 
# Par contre, la fonction de prédiction existe et elle renvoit  la probabilité de Y = 0 et Y = 1 

# In[25]:


logistic.predict_proba(data_test[train_cols])


# In[26]:


# pour calculer la moyenne du taux de défaut prédit 
logistic.predict_proba(data_test[train_cols]).mean(axis=0)

# on trouve à nouveau 22%